package cic.wsd.disambiguation;

import java.util.ArrayList;


import cic.wordnet.ParsedSynset;
import cic.wordnet.WordNet;
import cic.wsd.semcor.AmbiguousWord;
import cic.wsd.testing.Decision;
/**
 * Conceptual density algorithm as proposed by Agirre and Rigau in "Word Sense Disambiguation using Conceptual Density".
 * We also used "Map-based vs. knowledge-based toponym disambiguation" from Buscaldi and Rosso as additional reference.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class ConceptualDensity extends WSDAlgorithm {

	 @Override
		public Decision disambiguate(AmbiguousWord target, ArrayList<AmbiguousWord> window)
		{
			ArrayList<ParsedSynset> senses=target.getSenses();
			Decision decision=new Decision(target,window);
			int i;
			
			double w;
			for(i=0;i<senses.size();i++)
			{
				w=0.0d;
				double m=0.0d;
				
				ArrayList<String> bow=new ArrayList<String>();
				bow.add(target.getLemma());
				for(AmbiguousWord winword:window)
					bow.add(winword.getLemma());
				ArrayList<String> dwords=new ArrayList<String>();
				ArrayList<ArrayList<String>> cdwords=this.getTree(target.getSenses().get(i));
				for(ArrayList<String> synsets:cdwords)
				{
					double t=0.0d;
					for(String word:synsets)
					{
						for(String bword:bow)
						{
							if(word.equals(bword))
							{
								t+=1.0d;
								dwords.add(word);
								break;
							}
						}
						if(t>0.0d)
							break;
					}
					m+=t;
				}
				w=Math.pow(m,0.1)*(m/cdwords.size());
				decision.setSense(i, w, dwords);
			}
			decision.calculateAnswer();
			return decision;
		}
	/**
	 * Method for extracting the tree neighborhood of a sense.
	 * @param sense The target sense.
	 * @return An ArrayList containing ArrayLists with the words of each neighbor synset.
	 */
	private ArrayList<ArrayList<String>> getTree(ParsedSynset sense) {
		ArrayList<ArrayList<String>> tree=new ArrayList<ArrayList<String>>();
		ArrayList<String> hyps=sense.getSyds("@");
		hyps.addAll(sense.getSyds("@i"));
		ArrayList<String> aux;
		for(String hyp:hyps)
		{
			ParsedSynset ps=WordNet.getSynset(hyp);
			tree.add(ps.getSynonyms());
			aux=ps.getSyds("~");
			aux.addAll(ps.getSyds("~i"));
			for(String hypo:aux)
			{
				ParsedSynset ps2=WordNet.getSynset(hypo);
				if(!ps2.equals(sense))
				{
					tree.add(ps2.getSynonyms());
				}
			}
			
		}
		
		
		return tree;
	}

	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return "ConceptualDensity";
	}

}
