package cic.wsd.semcor;

import cic.wordnet.WordNet;
import cic.wsd.semcor.pruning.Pruning;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Class for loading a SEMCOR XML file.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class Input {
/**
 * An ArrayList with the ambiguous words (words found in WordNet) of the SEMCOR document.	
 */
ArrayList<AmbiguousWord> ambiguousWords;
/**
 * A simple index indicating the position (appearances) of each lemma on the document.
 */
HashMap<String,ArrayList<Integer>>Index;
/**
 * This file's name.
 */
String name;
/**
 * Returns an ArrayList with all the open-class words of this document.
 * @return ambiguousWords
 */
public ArrayList<AmbiguousWord> getAmbiguousWords() {
	return ambiguousWords;
}
/**
 * Returns a HashMap indicating the position (appearances) of each lemma on the document.
 * @return Index
 */
public HashMap<String, ArrayList<Integer>> getIndex() {
	return Index;
}

/**
 * Parse a SEMCOR file.
 * @param file The file to be parsed.
 * @throws Exception
 */
public Input(File file, ArrayList<Pruning> pruningList)throws Exception
{
	
	DocumentBuilderFactory fact = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = fact.newDocumentBuilder();
	Document test=builder.parse(file);
	this.name=file.getName();
	NodeList words=test.getElementsByTagName("wf");
	this.ambiguousWords=new ArrayList<AmbiguousWord>(words.getLength());
	int index=0;
	for(int i=0;i<words.getLength();i++)
	{
		Element word=(Element)words.item(i);
		String pos=word.getAttribute("pos").substring(0,1).toUpperCase();
		if(word.getAttribute("cmd").equals("done"))
			if(pos.equals("N")||pos.equals("V")||pos.equals("R")||pos.equals("J")||pos.equals("W"))
					if(WordNet.Morphy(word.getAttribute("lemma"), pos).size()>0)
					{
						AmbiguousWord nword=new AmbiguousWord(word.getAttribute("lemma"),word.getAttribute("pos").substring(0,1).toUpperCase(),word.getAttribute("wnsn"),index);
						for(Pruning prune:pruningList)
							prune.prune(nword);
						this.ambiguousWords.add(nword);
						index++;
					}
	}
	this.ambiguousWords.trimToSize();
	this.Indexing();
	this.calculateTF();
}

/**
 * Calculates the TF value for all the ambiguous words in this document. 
 */
private void calculateTF() {
	for(AmbiguousWord word:this.ambiguousWords)
	{
		ArrayList<Integer> aux=this.Index.get(word.getLemma());
		word.setTf((double)aux.size()/(double)this.ambiguousWords.size());
	}
}
/**
 * Creates the index of lemma appearances.
 */
private void Indexing()
{
	Index=new HashMap<String,ArrayList<Integer>>();
	int i=0;
	for(AmbiguousWord word:this.ambiguousWords)
	{
		if(Index.containsKey(word.getLemma()))
		{
			Index.get(word.getLemma()).add(new Integer(i));
		}
		else
		{
			ArrayList<Integer> aux=new ArrayList<Integer>(5);
			aux.add(new Integer(i));
			Index.put(word.getLemma(), aux);
		}
		i++;
	}
}

/**
 * Returns the name of this file.
 */
public String toString()
{
	return name;
}
}
