package cic.wsd.semcor.pruning;

import java.util.ArrayList;

import cic.wsd.semcor.AmbiguousWord;
/**
 * Generic template for pruning methods. 
 * Pruning methods are used for removing senses from words.
 * This is used for testing the effects of disambiguating between less senses.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public abstract class Pruning {
	/**
	 * Parameters for the pruning method.
	 */
	ArrayList<String> parameters;
	/**
	 * Name of the pruning method. Child classed should must set a value.
	 */
	String name;
	/**
	 * Instantiate a new pruning method.
	 * @param parameter Parameters for the pruning method. 
	 * @param name Name of the pruning method.
	 * @throws Exception
	 */
	public Pruning(String parameter, String name)throws Exception
	{
		this.name=name;
		String []values=parameter.split(",");
		this.parameters=new ArrayList<String>(values.length);
		for(String value:values)
		{
			this.parameters.add(value);
		}
	}
	/**
	 * Method for removing senses from a word.
	 * @param target Word to be pruned.
	 */
	public abstract void prune(AmbiguousWord target);
	public String toString()
	{
		return this.name;
	}
}
