package cic.wsd.testing;

import java.util.ArrayList;

/**
 * Utility class containing statistics measures.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class Measures {
	/**
	 * Calculates precision values for several test results.
	 * @param summaries An ArrayList containing test results.
	 * @return An array with the calculated precision values in the following order: {noun, verb, adjective, adverb, global}
	 */
	public static double[] Precision(ArrayList<Summary> summaries)
	{
		double []p={0.0,0.0,0.0,0.0,0.0};
		double []attempts={0.0,0.0,0.0,0.0,0.0};
		double []hits={0.0,0.0,0.0,0.0,0.0};
		for(Summary summary:summaries)
		{
			for(int i=0;i<5;i++)
			{
				attempts[i]+=(double)summary.getAttempts()[i];
				hits[i]+=(double)summary.getHits()[i];
			}
		}
		for(int i=0;i<5;i++)
		{
			p[i]=hits[i]/attempts[i];
			p[i]*=100.0;
		}
		return p;
	}
	/**
	 * Calculates precision values for several test results.
	 * @param summaries An ArrayList containing test results.
	 * @return An array with the calculated recall values in the following order: {noun, verb, adjective, adverb, global}
	 */
	public static double[] Recall(ArrayList<Summary> summaries)
	{
		double []r={0.0,0.0,0.0,0.0,0.0};
		double []counts={0.0,0.0,0.0,0.0,0.0};
		double []hits={0.0,0.0,0.0,0.0,0.0};
		for(Summary summary:summaries)
		{
			for(int i=0;i<5;i++)
			{
				counts[i]+=(double)summary.getCounts()[i];
				hits[i]+=(double)summary.getHits()[i];
			}
		}
		for(int i=0;i<5;i++)
		{
			r[i]=hits[i]/counts[i];
			r[i]*=100.0;
		}
		return r;
	}
	/**
	 * Calculates F1-measure values for several test results.
	 * @param summaries An ArrayList containing test results.
	 * @return An array with the calculated F1-measure values in the following order: {noun, verb, adjective, adverb, global}
	 */
	public static double[]Fmeasure(ArrayList<Summary> summaries)
	{
		double[]f={0.0,0.0,0.0,0.0,0.0};
		double []p=Measures.Precision(summaries);
		double []r=Measures.Recall(summaries);
		for(int i=0;i<5;i++)
			if(r[i]>0.0)
				f[i]=2*(p[i]*r[i])/(p[i]+r[i]);
		return f;
	}
	/**
	 * Calculates precision values for test results from a document.
	 * @param summary Test results from a document.
	 * @return An array with the calculated precision values in the following order: {noun, verb, adjective, adverb, global}
	 */
	public static double[] Precision(Summary summary) {
		double[]p={0.0,0.0,0.0,0.0,0.0};
		
		for(int i=0;i<5;i++)
		{
				p[i]=(double)summary.getHits()[i]/(double)summary.getAttempts()[i];
				p[i]*=100.0;
		}
		return p;
	}
	/**
	 * Calculates recall values for test results from a document.
	 * @param summary Test results from a document.
	 * @return An array with the calculated recall values in the following order: {noun, verb, adjective, adverb, global}
	 */
	public static double[] Recall(Summary summary) {
		double[]r={0.0,0.0,0.0,0.0,0.0};
		for(int i=0;i<5;i++)
		{
			r[i]=(double)summary.getHits()[i]/(double)summary.getCounts()[i];
			r[i]*=100.0;
		}
		return r;
	}
	/**
	 * Calculates recall values for test results from a document.
	 * @param summary Test results from a document.
	 * @return An array with the calculated recall values in the following order: {noun, verb, adjective, adverb, global}
	 */
	public static double[] Fmeasure(Summary summary) {
		double[]f={0.0,0.0,0.0,0.0,0.0};
		double []p=Measures.Precision(summary);
		double []r=Measures.Recall(summary);
		for(int i=0;i<5;i++)
			if(r[i]>0.0)
				f[i]=2*(p[i]*r[i])/(p[i]+r[i]);
		return f;
	}
}
