package cic.wsd.testing;

import java.util.ArrayList;
import java.util.HashMap;

import cic.wordnet.WordNet;
import cic.wsd.semcor.Util;
/**
 * Class containing results from a single test over a single file.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class Summary {
	/**
	 * Number of {nouns, verbs, adjectives, adverbs, open-class words}
	 */
	int []counts={0,0,0,0,0};
	/**
	 * Number of correct answers over {nouns, verbs, adjectives, adverbs, open-class words}
	 */
	double []hits={0,0,0,0,0};
	/**
	 * Number of attempts over {nouns, verbs, adjectives, adverbs, open-class words}
	 */
	int []attempts={0,0,0,0,0};
	/**
	 * Lemmas used for making decisions. int[] contains the index of the problem solved.
	 */
	HashMap<String,int[]> decisionLemmas;
	/**
	 * The lemmas used for making decisions.
	 * @return decisionLemmas
	 */
	public HashMap<String, int[]> getDecisionLemmas() {
		return decisionLemmas;
	}

	/**
	 * Summarizes the decisions taken by an algorithm.
	 * @param decisions Decisions made by a single approach.
	 */
	public Summary(ArrayList<Decision> decisions)
	{
		decisionLemmas=new HashMap<String,int[]>();
		for(Decision d:decisions)
		{
			
			this.increaseCounter(this.counts,d.getTarget().getPOS());
			if(d.isAttempted())
			{
				if(d.getTarget().getSenses().size()>0)
					this.extractLemmas(d);
				this.increaseCounter(this.attempts,d.getTarget().getPOS());
				if(d.getScore()>0.0)
				{
					this.increaseCounter(this.hits,d.getTarget().getPOS(),d.getScore());
				}
			}
		}
	}
	/**
	 * Extract the lemmas used for making a decision.
	 * @param decision The target decision.
	 */
	private void extractLemmas(Decision decision) {
		
			
			for(int answer:decision.getAnswers())
			{
				ArrayList<String> words=new ArrayList<String>();
				words=Util.removeDuplicates(decision.getDecisionWords(answer));
				words.trimToSize();
				if(words.size()>0)
				{
					for(String word:words)
					{
						if(this.decisionLemmas.containsKey(word))
						{
							int []aux=this.decisionLemmas.get(word);
							aux[0]++;
							if(decision.isCorrect(answer))
								aux[1]++;
						}
						else
						{
							int []aux=new int[2];
							aux[0]= 1;
							if(decision.isCorrect(answer))
								aux[1]=1;
							else
								aux[1]=0;	
							this.decisionLemmas.put(word,aux);
						}
				
					}
				}
		
			}
	}
	/** 
	 * Utility method for increasing a counter.
	 * @param counter The counter array to increase to.
	 * @param pos The POS counter to increase to.
	 */
	private void increaseCounter(int []counter,String pos)
	{
		counter[4]++;
		counter[WordNet.getPOS(pos)]++;
	}
	/** 
	 * Utility method for increasing a counter.
	 * @param counter The counter array to increase to.
	 * @param pos The POS counter to increase to.
	 */
	private void increaseCounter(double []counter,String pos,double score)
	{
		counter[4]+=score;
		counter[WordNet.getPOS(pos)]+=score;
	}
	/**
	 * Returns the counts.
	 * @return counts
	 */
	public int[] getCounts() {
		return counts;
	}
	/**
	 * Returns the hits.
	 * @return hits
	 */
	public double[] getHits() {
		return hits;
	}
	/**
	 * Returns the attemps.
	 * @return attempts
	 */
	public int[] getAttempts() {
		return attempts;
	}
}
