package cic.wsd.testing;

import java.io.File;
import java.lang.reflect.Constructor;
import java.util.ArrayList;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import cic.wordnet.WordNet;
import cic.wsd.disambiguation.WSDAlgorithm;
import cic.wsd.semcor.Input;
import cic.wsd.semcor.pruning.FirstSenses;
import cic.wsd.semcor.pruning.NthSenseOnly;
import cic.wsd.semcor.pruning.Pruning;
import cic.wsd.semcor.pruning.RemoveNthSense;
import cic.wsd.windowing.Condition;

/**
 * Class used to run experiments specified in an XML file.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class TestSet {

	/**
 * Loads a configuration file and prepares everything for just running the tests. 
 * @param XMLfile Configuration file.
 * @throws Exception
 */
public static void runTests(String XMLfile)throws Exception
{
	//Load the XML
	DocumentBuilderFactory fact = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder = fact.newDocumentBuilder();
	Document testset=builder.parse(XMLfile);
	Element root=testset.getDocumentElement();
	Element data=(Element)root.getElementsByTagName("dict").item(0);
	//Load wordnet and stuff
	WordNet.setPath("CICWN/");
	WordNet.loadDataBase(data.getAttribute("sources"));
	String sources=data.getAttribute("sources");
	//Load the test sets
	System.out.println("Loading test sets");
	data=(Element)root.getElementsByTagName("testbed").item(0);
	String senses=data.getAttribute("senses");
	ArrayList<Pruning> pruningms=getPruningMethods(senses);
	NodeList docs=data.getElementsByTagName("docs");
	ArrayList<String> prefixes=new ArrayList<String>(docs.getLength());
	ArrayList<String> testsetsnames=new ArrayList<String>(docs.getLength());
	ArrayList<ArrayList<Input>> testsets=new ArrayList<ArrayList<Input>>(docs.getLength());
	for(int i=0;i<docs.getLength();i++)
	{
		Element doc=(Element)docs.item(i);
		prefixes.add(doc.getAttribute("prefix"));
		testsetsnames.add(doc.getAttribute("src"));
		testsets.add(parseInputs(doc.getAttribute("src"),pruningms));
	}
	System.out.println("Runing experiments");
	NodeList xlsfiles=root.getElementsByTagName("xls");
	
	for(int f=0;f<xlsfiles.getLength();f++)
	{
		Element xls=(Element)xlsfiles.item(f);
		String src=xls.getAttribute("src");
		String split[]=src.split("/");
		String path="";
		for(int x=0;x<split.length-1;x++)
			path+=split[x]+"/";
		System.out.println("Loading tests for "+src);
			
		ArrayList<String> filenames=new ArrayList<String>(prefixes.size());
		for(String prefix:prefixes)
		{
			filenames.add(path+prefix+split[split.length-1]);
		}
		boolean detailed=Boolean.parseBoolean(xls.getAttribute("detail"));
		
		
		NodeList testsn=xls.getElementsByTagName("test");
		ArrayList<ArrayList<Test>> tests=new ArrayList<ArrayList<Test>>(testsets.size());
		
		for(int j=0;j<testsets.size();j++)
		{
			tests.add(new ArrayList<Test>());
		}
		for(int t=0;t<testsn.getLength();t++)
		{
			Element test=(Element)testsn.item(t);
			data=(Element)test.getElementsByTagName("algorithm").item(0);
			WSDAlgorithm algorithm=(WSDAlgorithm)Class.forName("cic.wsd.disambiguation."+data.getAttribute("disambiguation")).newInstance();
			WSDAlgorithm backoff=null;
			if(!data.getAttribute("backoff").equals("none"))
				backoff=(WSDAlgorithm)Class.forName("cic.wsd.disambiguation."+data.getAttribute("backoff")).newInstance();
			WSDAlgorithm tie=null;
			if(!data.getAttribute("tie").equals("none"))
				tie=(WSDAlgorithm)Class.forName("cic.wsd.disambiguation."+data.getAttribute("tie")).newInstance();
			int windowSize=Integer.parseInt(data.getAttribute("windowSize"));
			NodeList nconditions=test.getElementsByTagName("condition");
			ArrayList<Condition> conditions=getConditions(nconditions);
			int j=0;
			for(ArrayList<Input> inputs:testsets)
			{
				ArrayList<Test>ts=tests.get(j);
				ts.add(new Test(inputs,  algorithm, windowSize, backoff, testsetsnames.get(j), tie, conditions, senses, sources));
				j++;
			}
		}
		int j=0;
		for(ArrayList<Test> testX:tests)
		{
			for(Test test:testX)
				test.run();
			XLSWriter.writeXLS(new File(filenames.get(j)), testX, detailed);
			j++;
		}
	}
}
/**
 * Retrieves the conditions from the XML configuration file.
 * @param nconditions Conditions nodes.
 * @return The specified conditions.
 * @throws Exception
 */
private static ArrayList<Condition> getConditions(NodeList nconditions) throws Exception{
		ArrayList<Condition> conditions=new ArrayList<Condition>(nconditions.getLength());
		for(int i=0;i<nconditions.getLength();i++)
		{
			Element condition=(Element)nconditions.item(i);
			if(!condition.getAttribute("type").equals("none"))
			{
				String split[]=condition.getAttribute("type").split(":");
				String name=split[0];
				@SuppressWarnings("unchecked")
				Constructor<Condition> constructor=(Constructor<Condition>) Class.forName("cic.wsd.windowing."+name).getConstructor(String.class);
				
				String parameter="";
				if(split.length>1)
				{
					parameter=split[1];
				}
				Condition con=(Condition)constructor.newInstance(parameter);
				conditions.add(con);
			}
		}
		return conditions;
	}
/**
 * Utility method for creating the Inputs for a source.
 * @param source The folder containing the test files.
 * @param pruningList The pruning methods to be applied.
 * @return An ArrayList with all the new Input instances.
 * @throws Exception
 */
private static ArrayList<Input> parseInputs(String source,ArrayList<Pruning> pruningList) throws Exception {
		ArrayList<File> files=WordNet.getAllFiles(new File(source));
		ArrayList<Input> inputs=new ArrayList<Input>(files.size());
		for(File file:files)
			inputs.add(new Input(file,pruningList));
		return inputs;
	}
/**
 * Retrieves the pruning methods from the XML configuration file.
 * @param senses The senses to be retrieved.
 * @return the pruning methods
 * @throws Exception
 */
private static ArrayList<Pruning> getPruningMethods(String senses) throws Exception {
		ArrayList<Pruning> pruningms=new ArrayList<Pruning>(1);
		if(senses.startsWith("+"))
			pruningms.add(new FirstSenses(senses.replace("+", "")));
		if(senses.startsWith("*"))
			pruningms.add(new NthSenseOnly(senses.replace("*", "")));
		if(senses.startsWith("-"))
			pruningms.add(new RemoveNthSense(senses.replace("-", "")));
		
		return pruningms;
	}
public static void main(String args[]) throws Exception
{
	/*Thing to do
	 * 1) Fix TestSet. 
	 * 2) Fix test. 
	 */
	String test;
	if(args.length>0)
	{
		test=args[0];
	}
	else
	{
		test="config.xml";
	}
	TestSet.runTests(test);
	System.out.println("Finished...");
}

}
