package cic.wsd.windowing;
import java.util.ArrayList;

import cic.wsd.semcor.AmbiguousWord;
/**
 * Generic template for a window selection condition.
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public abstract class Condition {
	/**
	 * Parameters for this condition.
	 */
	ArrayList<String> parameters;
	/**
	 * Name of this condition.
	 */
	String name;
	/**
	 * Instantiates a new condition
	 * @param parameter Parameters for this condition in the format "[parameter1[,parameterN]*]?"
	 * @param name Name of this condition.
	 * @throws Exception
	 */
	public Condition(String parameter,String name)throws Exception
	{
		this.name=name;
		String []values=parameter.split(",");
		this.parameters=new ArrayList<String>(values.length);
		for(String value:values)
		{
			this.parameters.add(value);
		}
	}
	/**
	 * Function for telling if a word could be added to the window.
	 * @param target Word to be disambiguated.
	 * @param possibleWord Word to be added to the window.
	 * @param window Current window.
	 * @return True, if the word could be added to the window.
	 */
	public abstract boolean satisfiesCondition(AmbiguousWord target,AmbiguousWord possibleWord, ArrayList<AmbiguousWord> window);
	public String toString()
	{
		return name;
	}
}
