package cic.wsd.windowing;

import java.util.ArrayList;

import cic.wsd.semcor.AmbiguousWord;
import cic.wsd.semcor.Util;
/**
 *Extracted from the paper " Evaluating Variants of the Lesk Approach for Disambiguating Words". 
 *J is a value in [0.0,1.0] that acts as a 
 *threshold for creating lexical chains (A lower value will allow an 
 *easy integration to the lexical chain). Only words that form a 
 *lexical chain with the target word will be selected. 
 * @author Francisco Viveros-Jim&eacute;nez
 *
 */
public class VascilescuLexicalChain extends Condition {
	/**
	 * 
	 * @param parameter The Jaccard score value to be used as a threshold.
	 * @throws Exception
	 */
	public VascilescuLexicalChain(String parameter) throws Exception {
		super(parameter,"VascilescuLexicalChain");
		J=Double.parseDouble(this.parameters.get(0));
		// TODO Auto-generated constructor stub
	}

	/**
	 * Margin for the Jaccard similarity coefficient used to measure similarity.
	 */
	double J;
	
	
	/**
	 * Jaccard similarity coefficient computation between two bag of words.
	 * @param targetWords Bag of words of the base word.
	 * @param possibleWords Bag of words of the possible new word of the lexical chain.
	 * @return Jaccard similarity coefficient.
	 */
	public double Jaccard(ArrayList<String> targetWords,
			ArrayList<String> possibleWords) {
		ArrayList<String> union= new ArrayList<String>();
		union.addAll(targetWords);
		union.addAll(possibleWords);
		union=Util.removeDuplicates(union);
		double intersection=0.0d;
		for(String word:targetWords)
			if(possibleWords.contains(word))
				intersection+=1.0d;
				
		return intersection/((double)union.size());
	}


	@Override
	public boolean satisfiesCondition(AmbiguousWord target,
			AmbiguousWord possibleWord, ArrayList<AmbiguousWord> window) {
		
		ArrayList<String> targetWords=new ArrayList<String>();
		targetWords.addAll(target.getPossibleSynonyms());
		targetWords.addAll(target.getPossibleHypernyms());
		ArrayList<String> possibleWords=new ArrayList<String>();
		possibleWords.addAll(possibleWord.getPossibleSynonyms());
		possibleWords.addAll(possibleWord.getPossibleHypernyms());
		return (this.Jaccard(targetWords,possibleWords)>J);
	}

}